extends CharacterBody2D


@export_range(1, 10, 0.2) var acceleration: float = 5
@export_range(100, 300, 2) var max_speed: int = 200
@export_range(50, 150, 0.99) var deceleration: int = 100
@export_range(1, 10, 0.2) var rotation_speed: float = 5
@export var projectile: PackedScene = preload("res://prefabs/laser/laser.tscn")

@onready var shot_timer: Timer = $ShotTimer
@onready var fire_nose: Marker2D = $FireNose


func _unhandled_input(_event: InputEvent) -> void:
	if Input.is_action_just_pressed("fire") and shot_timer.is_stopped():
		var obj := projectile.instantiate() as Node2D
		if "fire_rate" in obj:
			obj.global_position = fire_nose.global_position
			obj.rotation = rotation
			get_parent().add_child(obj)
			shot_timer.start(obj.fire_rate)
		else:
			push_error("Projectile does not have a Fire Rate, unable to fire")
			obj.queue_free()


func _physics_process(delta: float) -> void:
	var target_rotation: float = global_position.angle_to_point(get_global_mouse_position())
	rotation = rotate_toward(rotation, target_rotation, rotation_speed * delta)
	
	var input_vector := Vector2(Input.get_action_strength("thrust"), 0)
	velocity += input_vector.rotated(rotation) * acceleration
	velocity = velocity.limit_length(max_speed)
	
	if input_vector.is_equal_approx(Vector2.ZERO):
		velocity = velocity.move_toward(Vector2.ZERO, deceleration * delta)
	move_and_slide()


func _on_area_2d_body_entered(body: Node2D) -> void:
	if body is Asteroid:
		Relay.ship_hit.emit()
