extends Node2D
class_name Game

@export var asteroid_prefab: PackedScene = preload("res://prefabs/asteroid/asteroid.tscn")
@export var max_asteroids: int = 10
@export var asteroid_types: Array[AsteroidResource]
@export var reticle: Resource = preload("res://assets/textures/crossair_red.png")
var current_score: int = 0
var cursor: Resource = load(ProjectSettings.get_setting("display/mouse_cursor/custom_image"))
@onready var player: CharacterBody2D = $Player
@onready var camera: Camera2D = $Camera
@onready var asteroid_parent: Node2D = $AsteroidParent
@onready var path_follow_2d: PathFollow2D = $Camera/Path2D/PathFollow2D


func _ready() -> void:
	Relay.game_start.connect(on_game_start)
	Relay.ship_hit.connect(on_game_over)
	Relay.gain_point.connect(on_gain_point)
	Relay.current_score.emit(current_score)


func on_gain_point() -> void:
	current_score += 1
	Relay.current_score.emit(current_score)


func _on_spawn_timer_timeout() -> void:
	var obj := asteroid_prefab.instantiate() as Asteroid
	if asteroid_prefab and asteroid_parent.get_child_count() < max_asteroids:
		#TODO: randomly select asteroid type from array, with weights?
		obj.asteroid_type = asteroid_types[ randi() % asteroid_types.size() ]
		path_follow_2d.progress_ratio = randf()
		var spawn_position: Vector2 = camera.get_screen_center_position() + path_follow_2d.position
		var target_position := Vector2.ONE.rotated(randf_range(0, TAU)) * randf_range(40,100)
		target_position += player.global_position
		obj.global_position = spawn_position
		var rng_force = randf_range(obj.asteroid_type.speed_min_max.x, obj.asteroid_type.speed_min_max.y)
		obj.start_force = Vector2(target_position - spawn_position).normalized() * rng_force
		asteroid_parent.add_child(obj)
	elif not asteroid_prefab:
		push_error("Object is not Asteroid Class, unable to spawn asteroid")


#TODO: refactor this, possible signal changes
func on_game_start() -> void:
	Input.mouse_mode = Input.MOUSE_MODE_CONFINED
	var mouse_pos: Vector2 = get_viewport_rect().size / 2
	mouse_pos.y = mouse_pos.y / 1.5
	Input.warp_mouse(mouse_pos)
	Input.set_custom_mouse_cursor(reticle, Input.CURSOR_ARROW, Vector2(16,16))


func on_game_over() -> void:
	Input.mouse_mode = Input.MOUSE_MODE_VISIBLE
	Input.set_custom_mouse_cursor(cursor)
	get_tree().paused = true
	Relay.game_over.emit()
